@@ -40,21 +40,24 @@
 GPtrArray*
 tracker_data_query_rdf_type (gint id)
 {
-	TrackerDBCursor *cursor;
+	TrackerDBCursor *cursor = NULL;
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
 	GPtrArray *ret = NULL;
+	GError *error = NULL;
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdf:type\") "
 	                                              "FROM \"rdfs:Resource_rdf:type\" "
 	                                              "WHERE ID = ?");
 
-	tracker_db_statement_bind_int (stmt, 0, id);
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+	if (stmt) {
+		tracker_db_statement_bind_int (stmt, 0, id);
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
 
@@ -63,7 +66,7 @@ tracker_data_query_rdf_type (gint id)
 		 * function is called fairly often) */
 
 		ret = g_ptr_array_sized_new (20);
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, &error)) {
 			const gchar *class_uri;
 			TrackerClass *cl;
 
@@ -78,33 +81,54 @@ tracker_data_query_rdf_type (gint id)
 		g_object_unref (cursor);
 	}
 
+	if (G_UNLIKELY (error)) {
+		g_critical ("Could not query RDF type: %s\n", error->message);
+		g_error_free (error);
+
+		if (ret) {
+			g_ptr_array_free (ret, FALSE);
+			ret = NULL;
+		}
+	}
+
 	return ret;
 }
 
 gint
 tracker_data_query_resource_id (const gchar *uri)
 {
-	TrackerDBCursor *cursor;
+	TrackerDBCursor *cursor = NULL;
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
+	GError *error = NULL;
 	gint id = 0;
 
 	g_return_val_if_fail (uri != NULL, 0);
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT ID FROM Resource WHERE Uri = ?");
-	tracker_db_statement_bind_text (stmt, 0, uri);
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		tracker_db_statement_bind_text (stmt, 0, uri);
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor, NULL);
-		id = tracker_db_cursor_get_int (cursor, 0);
+		if (tracker_db_cursor_iter_next (cursor, &error)) {
+			id = tracker_db_cursor_get_int (cursor, 0);
+		}
+
 		g_object_unref (cursor);
 	}
 
+	if (G_UNLIKELY (error)) {
+		g_critical ("Could not query resource ID: %s\n", error->message);
+		g_error_free (error);
+	}
+
 	return id;
 }
 
